# Generated by Django 4.2.5 on 2023-10-13 02:39

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0004_content_types_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="embeddings",
            name="corpus_id",
            field=models.UUIDField(default=uuid.uuid4, editable=False),
        ),
    ]
