# Generated by Django 4.2.11 on 2024-06-21 04:17

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0046_khojuser_email_verification_code_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="entry",
            name="file_type",
            field=models.CharField(
                choices=[
                    ("image", "Image"),
                    ("pdf", "Pdf"),
                    ("plaintext", "Plaintext"),
                    ("markdown", "Markdown"),
                    ("org", "Org"),
                    ("notion", "Notion"),
                    ("github", "Github"),
                    ("conversation", "Conversation"),
                    ("docx", "Docx"),
                ],
                default="plaintext",
                max_length=30,
            ),
        ),
    ]
