"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[8086],{98086:function(e,r,t){t.d(r,{diagram:function(){return c}});var n=t(40330),a=t(38111),s=t(20755),i=t(54327),d={parse:(0,s.eW)(async e=>{let r=await (0,i.Qc)("info",e);s.cM.debug(r)},"parse")},o={version:n.X.version+""},c={parser:d,db:{getVersion:(0,s.eW)(()=>o.version,"getVersion")},renderer:{draw:(0,s.eW)((e,r,t)=>{s.cM.debug("rendering info diagram\n"+e);let n=(0,a.P)(r);(0,s.v2)(n,100,400,!0),n.append("g").append("text").attr("x",100).attr("y",40).attr("class","version").attr("font-size",32).style("text-anchor","middle").text(`v${t}`)},"draw")}}}}]);